# This Rscript creates an extension_config.tsv file that defines which qualifiers (extensions in pombase lingo) can go with which type of annotation, specified in 'allowed_qualifiers.tsv'
# The list of allowed qualifiers comes as an array of namespaces
# The annotation types are defined as the top term IDs for the type of annotation
  # phenotypic class - FBcv:0000347
  # anatomy/manifests uses two ontologies and thus needs two ids:
    # flybase anatomy - FBbt:10000000
    # GO CC - GO:0005575.
# extension_config.tsv only uses IDs and, therefore, qualifier namespaces must be converted into their topterm IDs, whcih requires screening the FBcv obo file.

#set work directory as canto-space
setwd("./")
getwd()

# 1 - pick the default namespaces from the FBcv ontology 
##tempFBcvobo.txt is a derivation of the original FBcv obo file, stripped of text, etc, and with ':' replaced by '\t' so that R can read it as a 2 colmun table
obo<-read.table("./extension_config-Rscript/tempFBcvobo.txt", sep='\t', fill=TRUE, quote = "")
obo[,1]<-as.character(obo[,1])
obo[,2]<-as.character(obo[,2])
ids<-which(obo[,1]=="id")
idl<-length(ids)
a<-0

#Create a working table for term-term relationships, etc
Df<-data.frame(matrix(ncol=6), stringsAsFactors = FALSE)
colnames(Df)<-c("cv term","name","namespace","parent term","top namespace", "topmost namespace?")

#loop to remove obsolete terms, and move CV, name, namespace, and relation into separate columns
for (i in 1:(idl-1)) {
  obo1<-obo[(ids[i]:ids[i+1]),]
  obsolete<-which(obo1[,1]=="is_obsolete", obo1[,2]=="true")
    if (length(obsolete)==0){
    a=a+1
    Df[a,1]=obo[ids[i],2]
    Df[a,2]=obo[ids[i]+1,2]
    b<-which(obo1[,1]=="namespace")
    if (length(b)==1){
      Df[a,3]<-obo[(ids[i]+b-1),2]
    }
    c<-which(obo1[,1]=="is_a")
    if (length(c)==1){
      Df[a,4]<-obo[(ids[i]+c-1),2]
    }
  }
}

#same loop as above but only for the last term - removes obsolete terms, and moves CV, name, namespace, and relation into sep columns
obo1<-obo[(ids[i+1]:max(ids)),]
obsolete<-which(obo1[,1]=="is_obsolete", obo1[,2]=="true")
if (length(obsolete)==0){
  a=a+1
  Df[a,1]=obo[ids[i+1],2]
  Df[a,2]=obo[ids[i+1]+1,2]
  b=which(obo1[,1]=="namespace")
  if (length(b)==1){
  Df[a,3]<-obo[(ids[i+1]+b),2]
  }
  c<-which(obo1[,1]=="is_a")
  if (length(c)==1){
  Df[a,4]<-obo[(ids[i+1]+c-1),2]
  }
}

#for the is_a field, this will crop the CV term (e.g. FBcv:0000001 -> 4 letters+colon+7 numbers, adding up to 12 characters)
Df[,4]<-substr(Df[,4], 1, 12)

#Loop to pick the top-most namespace, using the is_a parent-child relationships
## 1 - set top namespace
Df[which(Df[,1]=="FBcv:0000000"),3]<-"FlyBase_miscellaneous_CV"
## 2 - if, existing, pick the current namespace
for(j in 1:nrow(Df)){
  if(is.na(Df[j,3])==FALSE){Df[j,5]<-Df[j,3]}
}
## 3 - otherwise, go through all childs of a term and, if child does not have a namespace, pick the parent namespace
rm(j)
for(j in 1:nrow(Df)){
  childs<-which(Df[,4]==Df[j,1])
  if(length(childs)>0){
    for(k in 1:length(childs)){
      if(is.na(Df[childs[k],5])==FALSE){}
      else{if(is.na(Df[j,3])==FALSE){
      Df[childs[k],5]<-Df[j,3]}
        else{
          if(is.na(Df[j,5])==FALSE){Df[childs[k],5]<-Df[j,5]}
        }
      }
    }
    rm(k)
    for(k in 1:length(childs)){
      if(is.na(Df[childs[k],5])==FALSE&is.na(Df[j,3])==FALSE&Df[childs[k],5]==Df[j,3]){
        Df[childs[k],6]<-"child"
      }
    }
  }
}


# create the template for the 'extension_config.tsv' file
Df1<-data.frame(matrix(ncol=8), stringsAsFactors = FALSE)
colnames(Df1)<-c("domain ID", "subset relation", "extension relation", "range ID", "Canto display text", "Help text", "cardinality", "role")

#read file with list of allowed qualifiers (on "allowed_qualifiers.tsv")
quallist<-read.table("./extension_config-Rscript/allowed_qualifiers.tsv", sep="\t", header=TRUE, stringsAsFactors=FALSE)

count<-1
#for each type of allowed qualifier, replace the namespace with corresponding top term with that namespace
for(m in 1:nrow(quallist)){
  phen<-unlist(strsplit(quallist[m,3],","))
  for(l in 1:length(phen)){
    phen[l]<-Df[which(Df[,3]==phen[l]&Df[,5]==phen[l]&is.na(Df[,6])==TRUE),1]
  }
  
  if(quallist[m,2]=='phenotype'){
    #if the annotations are for 'phenotypic class' (FBcv:0000347) then:
    # a) the single term 'progressive' shoud be allowed
    if(quallist[m,1]==Df[which(Df[,2]=="phenotypic class"),1]){
      Df1[count,1]<-quallist[m,1]
      phen<-append(phen, Df[which(Df[,2]=="progressive"), 1])
      Df1[count,4]<-paste(phen,collapse = '|')
      count<-count+1
    # b) and 'the 'fertility/sterility terms must not allow developmental stage terms
    ##once the ontology file is updated so that all viability/mortality/life-span terms are under a common parent term, this next bit should be updated
      viable<-c("viable","increased mortality","long lived")
      for (n in 1:length(viable)){ 
      viable[n]<-Df[which(Df[,2]==viable[n]),1]
      }
      #once the ontology file is updated to put term with decreased fertility under a common parent term (e.g. 'decreased fertily'), the next bit should be updated to refer to the new top term
      sterile<-c("fertile", "sterile","semi-sterile","semi-fertile")
      for (o in 1:length(sterile)){
        sterile[o]<-Df[which(Df[,2]==sterile[o]),1]
      }
      viablesterile<-c(viable,sterile)
      Df1[count,1]<-paste(quallist[m,1],"-is_a(",paste(viablesterile, collapse=")&is_a("),")",sep="")
      Df1[count,4]<-"FBdv:00007008"
      Df1[count,3]<-"dv_qual"
      Df1[count,5]<-"Developmental stage qualifier"
      Df1[count,7]<-"0,1"
      count<-count+1
    }
  else{
    Df1[count,1]<-quallist[m,1]
    Df1[count,4]<-paste(phen, collapse = '|')
    count<-count+1
    Df1[count,1]<-quallist[m,1]
    Df1[count,4]<-"FBdv:00007008"
    Df1[count,3]<-"dv_qual"
    Df1[count,5]<-"Developmental stage qualifier"
    Df1[count,7]<-"0,1"
    count<-count+1
  }
  }
  if(quallist[m,2]=='genetic interaction'){
    Df1[count,1]<-quallist[m,1]
    Df1[count,4]<-paste(phen, collapse = '|')
    Df1[count,3]<-"int_qual"
    Df1[count,5]<-"Interaction qualifier - ONLY USE FOR INTERACTIONS"
    count<-count+1
  }
}


for (j in 1:nrow(Df1)){
if (is.na(Df1[j,2])==TRUE){Df1[j,2]<-as.character("is_a")}
  else{}
if (is.na(Df1[j,3])==TRUE){Df1[j,3]<-as.character("qual")}
  else{}
if (is.na(Df1[j,5])==TRUE){Df1[j,5]<-as.character("Qualifier")}
  else{}
if (is.na(Df1[j,6])==TRUE){Df1[j,6]<-as.character("")}
  else{}
if (is.na(Df1[j,7])==TRUE){Df1[j,7]<-as.character("*")}
  else{}
if (is.na(Df1[j,8])==TRUE){Df1[j,8]<-as.character("user")}
  else{}
}

write.table(Df1, file = "./canto/extension_config.tsv", sep='\t', quote= FALSE, row.names = FALSE)